﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.DesktopEdition;

namespace WindowsFormsApplication1
{
    public partial class Form1 : Form
    {
        private InMemoryFeatureLayer selectedFeatureLayer;

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            winformsMap1.MapUnit = GeographyUnit.DecimalDegree;

            LayerOverlay lo = new LayerOverlay();
            winformsMap1.Overlays.Add(lo);

            selectedFeatureLayer = new InMemoryFeatureLayer();
            selectedFeatureLayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = LineStyles.Highway1;
            selectedFeatureLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            lo.Layers.Add(selectedFeatureLayer);

            winformsMap1.TrackOverlay.TrackMode = TrackMode.Line;
            winformsMap1.TrackOverlay.TrackEnded += TrackOverlay_TrackEnded;

            winformsMap1.CurrentExtent = new RectangleShape(-40, 30, 40, -10);

            winformsMap1.MapClick += winformsMap1_MapClick;
            winformsMap1.Refresh();
        }

        private void TrackOverlay_TrackEnded(object sender, TrackEndedTrackInteractiveOverlayEventArgs e)
        {
            winformsMap1.TrackOverlay.TrackShapeLayer.InternalFeatures.RemoveAt(winformsMap1.TrackOverlay.TrackShapeLayer.InternalFeatures.Count - 1);

            LineShape lineShape = e.TrackShape as LineShape;
            if (lineShape != null)
            {
                for (int i = 0; i < lineShape.Vertices.Count - 1; i++)
                {
                    LineShape tempSegment = new LineShape();
                    tempSegment.Vertices.Add(lineShape.Vertices[i]);
                    tempSegment.Vertices.Add(lineShape.Vertices[i + 1]);

                    winformsMap1.TrackOverlay.TrackShapeLayer.InternalFeatures.Add(new Feature(tempSegment));
                }
            }
        }

        private void winformsMap1_MapClick(object sender, MapClickWinformsMapEventArgs e)
        {
            if (winformsMap1.TrackOverlay.TrackMode == TrackMode.None)
            {
                foreach (var trackFeature in winformsMap1.TrackOverlay.TrackShapeLayer.InternalFeatures)
                {
                    PointShape closetPoint = trackFeature.GetShape().GetClosestPointTo(e.WorldLocation, winformsMap1.MapUnit);
                    double toleranceInPixel = 10;

                    if (closetPoint != null && ExtentHelper.GetScreenDistanceBetweenTwoWorldPoints(winformsMap1.CurrentExtent, closetPoint, e.WorldLocation, winformsMap1.Width, winformsMap1.Height) < toleranceInPixel)
                    {
                        selectedFeatureLayer.InternalFeatures.Clear();
                        selectedFeatureLayer.InternalFeatures.Add(trackFeature);

                        winformsMap1.Refresh();
                        break;
                    }
                }
            }
        }

        private void btnEdit_Click(object sender, EventArgs e)
        {
            selectedFeatureLayer.InternalFeatures.Clear();
            winformsMap1.TrackOverlay.TrackMode = TrackMode.None;

            winformsMap1.Refresh();
        }

        private void btnTrack_Click(object sender, EventArgs e)
        {
            selectedFeatureLayer.InternalFeatures.Clear();
            winformsMap1.TrackOverlay.TrackMode = TrackMode.Line;
        }

        private void btnDelete_Click(object sender, EventArgs e)
        {
            if (selectedFeatureLayer.InternalFeatures.Count > 0)
            {
                winformsMap1.TrackOverlay.TrackShapeLayer.InternalFeatures.Remove(selectedFeatureLayer.InternalFeatures[0]);
                selectedFeatureLayer.InternalFeatures.Clear();

                winformsMap1.Refresh();
            }
        }
    }
}